from pathlib import Path

import boto3
from moto import mock_s3

from pys3thon.client import S3Client


@mock_s3
def test_upload_and_delete_directory(tmpdir):
    tmpdir = Path(tmpdir)
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()

    output_dir = tmpdir / "output"
    (output_dir / "nested1" / "nested2").mkdir(parents=True, exist_ok=True)

    with open(output_dir / "file_1.txt", "w") as file1:
        # Write some content to the file
        file1.write("Hello, world!")

    with open(output_dir / "nested1" / "file_2.txt", "w") as file2:
        # Write some content to the file
        file2.write("Hello, world!")

    with open(output_dir / "nested1" / "nested2" / "file_3.txt", "w") as file3:
        # Write some content to the file
        file3.write("Hello, world!")

    s3_client.upload_directory(str(tmpdir), "test-bucket", "test-directory-1/")

    assert s3_client.check_if_exists_in_s3(
        bucket="test-bucket", key="test-directory-1/output/file_1.txt"
    )

    assert s3_client.check_if_exists_in_s3(
        bucket="test-bucket", key="test-directory-1/output/nested1/file_2.txt"
    )

    assert s3_client.check_if_exists_in_s3(
        bucket="test-bucket",
        key="test-directory-1/output/nested1/nested2/file_3.txt",
    )

    s3_client.delete_directory(
        "test-bucket", "test-directory-1/output/nested1"
    )

    assert s3_client.check_if_exists_in_s3(
        bucket="test-bucket", key="test-directory-1/output/file_1.txt"
    )

    assert not s3_client.check_if_exists_in_s3(
        bucket="test-bucket", key="test-directory-1/output/nested1/file_2.txt"
    )

    assert not s3_client.check_if_exists_in_s3(
        bucket="test-bucket",
        key="test-directory-1/output/nested1/nested2/file_3.txt",
    )



import io

import boto3
from moto import mock_s3

from pys3thon.client import S3Client


@mock_s3
def test_get_files_for_bucket_with_no_files_in_root_prefix():
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-1/subdirectory-1/test_fileobj-1.txt",
    )

    file_contents = s3_client.get_files_for_bucket_with_prefix(
        bucket="test-bucket", prefix="test-directory-1/"
    )

    assert len(file_contents) == 0


@mock_s3
def test_get_files_for_bucket_with_prefix_for_less_than_999_files():
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    for i in range(10):
        s3_client.upload_fileobj(
            io.BytesIO(b"my data stored as file object in RAM"),
            bucket="test-bucket",
            key=f"test-directory-1/test_fileobj-{i}.txt",
        )

    file_contents = s3_client.get_files_for_bucket_with_prefix(
        bucket="test-bucket", prefix="test-directory-1/"
    )

    assert len(file_contents) == 10
    assert sorted([file_content["Key"] for file_content in file_contents]) == [
        f"test-directory-1/test_fileobj-{i}.txt" for i in range(10)
    ]


@mock_s3
def test_get_files_for_bucket_with_prefix_for_more_than_999_files():
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    for i in range(1001):
        s3_client.upload_fileobj(
            io.BytesIO(b"my data stored as file object in RAM"),
            bucket="test-bucket",
            key=f"test-directory-1/test_fileobj-{i}.txt",
        )

    file_contents = s3_client.get_files_for_bucket_with_prefix(
        bucket="test-bucket", prefix="test-directory-1/"
    )

    assert len(file_contents) == 1001
    assert sorted(
        [file_content["Key"] for file_content in file_contents]
    ) == sorted(
        [f"test-directory-1/test_fileobj-{i}.txt" for i in range(1001)]
    )



import os
from pathlib import Path
from uuid import uuid4

import boto3
from moto import mock_s3

from pys3thon.client import S3Client


@mock_s3
def test_download_file(tmpdir):
    tmpdir = Path(tmpdir)
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    source_bucket = "test-bucket"
    source_key = f"{str(uuid4())}/Nepean.pdf"

    tmp_file_path = str(tmpdir / "Nepean.pdf")
    with open(tmp_file_path, "w") as file1:
        # Write some content to the file
        file1.write("Hello, world!")

    s3_client.upload_file(tmp_file_path, source_bucket, source_key)
    assert s3_client.check_if_exists_in_s3(source_bucket, source_key)

    try:
        file_name = source_key.split("/")[-1]
        s3_client.download(source_bucket, source_key, tmpdir / file_name)
        assert Path(tmpdir / file_name).exists()
    except Exception as e:
        assert False, str(e)


@mock_s3
def test_download_file_with_key_that_is_uri_encoded(tmpdir):
    tmpdir = Path(tmpdir)
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    source_bucket = "test-bucket"
    source_key = f"{str(uuid4())}/Nepean%20Legend(2).pdf"

    tmp_file_path = str(tmpdir / "Nepean.pdf")
    with open(tmp_file_path, "w") as file1:
        # Write some content to the file
        file1.write("Hello, world!")

    s3_client.upload_file(tmp_file_path, source_bucket, source_key)
    assert s3_client.check_if_exists_in_s3(source_bucket, source_key)

    try:
        file_name = source_key.split("/")[-1]
        s3_client.download(source_bucket, source_key, tmpdir / file_name)
        assert Path(tmpdir / file_name).exists()
    except Exception as e:
        assert False, str(e)


@mock_s3
def test_download_to_temporary_file(tmpdir):
    tmpdir = Path(tmpdir)
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    source_bucket = "test-bucket"
    source_key = f"{str(uuid4())}/Nepean.pdf"

    tmp_file_path = str(tmpdir / "Nepean.pdf")
    with open(tmp_file_path, "w") as file1:
        # Write some content to the file
        file1.write("Hello, world!")

    s3_client.upload_file(tmp_file_path, source_bucket, source_key)
    object_size = s3_client.get_object_size(source_bucket, source_key)
    assert s3_client.check_if_exists_in_s3(source_bucket, source_key)

    try:
        with s3_client.download_to_temporary_file(
            source_bucket, source_key
        ) as temp_file_path:
            assert Path(temp_file_path).name == source_key.split("/")[-1]
            assert Path(temp_file_path).exists()
            assert os.stat(temp_file_path).st_size == object_size
        assert Path(temp_file_path).exists() is False
    except Exception as e:
        assert False, str(e)


@mock_s3
def test_download_to_temporary_file_with_specified_file_name(tmpdir):
    tmpdir = Path(tmpdir)
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    source_bucket = "test-bucket"
    source_key = f"{str(uuid4())}/Nepean.pdf"

    tmp_file_path = str(tmpdir / "Nepean.pdf")
    with open(tmp_file_path, "w") as file1:
        # Write some content to the file
        file1.write("Hello, world!")

    s3_client.upload_file(tmp_file_path, source_bucket, source_key)
    object_size = s3_client.get_object_size(source_bucket, source_key)
    assert s3_client.check_if_exists_in_s3(source_bucket, source_key)

    try:
        with s3_client.download_to_temporary_file(
            source_bucket, source_key, file_name="some_file_name"
        ) as temp_file_path:
            assert Path(temp_file_path).name == "some_file_name"
            assert Path(temp_file_path).exists()
            assert os.stat(temp_file_path).st_size == object_size
        assert Path(temp_file_path).exists() is False
    except Exception as e:
        assert False, str(e)



import io

import boto3
from moto import mock_s3

from pys3thon.client import S3Client


@mock_s3
# flake8: noqa
def test_get_directories_for_bucket_with_prefix_recursively_works_on_bucket_root_with_one_directory():
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    test_fileobj = io.BytesIO(b"my data stored as file object in RAM")
    s3_client.upload_fileobj(
        test_fileobj,
        bucket="test-bucket",
        key="test-directory/test_fileobj.txt",
    )

    directories = s3_client.get_directories_for_bucket_with_prefix_recursively(
        bucket="test-bucket", prefix=None
    )
    assert directories == ["test-directory/"]


@mock_s3
# flake8: noqa
def test_get_directories_for_bucket_with_prefix_recursively_works_on_bucket_root_with_three_directories():
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-1/test_fileobj.txt",
    )
    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-2/test_fileobj.txt",
    )
    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-3/test_fileobj.txt",
    )

    directories = s3_client.get_directories_for_bucket_with_prefix_recursively(
        bucket="test-bucket", prefix=None
    )

    assert sorted(directories) == [
        "test-directory-1/",
        "test-directory-2/",
        "test-directory-3/",
    ]


@mock_s3
# flake8: noqa
def test_get_directories_for_bucket_with_prefix_recursively_works_on_bucket_root_directory_three_levels_deep():
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    test_fileobj = io.BytesIO(b"my data stored as file object in RAM")
    s3_client.upload_fileobj(
        test_fileobj,
        bucket="test-bucket",
        key="test-directory/a/b/c/test_fileobj.txt",
    )

    directories = s3_client.get_directories_for_bucket_with_prefix_recursively(
        bucket="test-bucket", prefix=None
    )
    assert sorted(directories) == [
        "test-directory/",
        "test-directory/a/",
        "test-directory/a/b/",
        "test-directory/a/b/c/",
    ]


@mock_s3
# flake8: noqa
def test_get_directories_for_bucket_with_prefix_recursively_works_on_bucket_root_with_three_directories_three_levels_deep():
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()

    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-1/a/b/c/test_fileobj.txt",
    )
    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-2/a/b/c/test_fileobj.txt",
    )
    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-3/a/b/c/test_fileobj.txt",
    )

    directories = s3_client.get_directories_for_bucket_with_prefix_recursively(
        bucket="test-bucket", prefix=None
    )

    assert sorted(directories) == [
        "test-directory-1/",
        "test-directory-1/a/",
        "test-directory-1/a/b/",
        "test-directory-1/a/b/c/",
        "test-directory-2/",
        "test-directory-2/a/",
        "test-directory-2/a/b/",
        "test-directory-2/a/b/c/",
        "test-directory-3/",
        "test-directory-3/a/",
        "test-directory-3/a/b/",
        "test-directory-3/a/b/c/",
    ]


@mock_s3
# flake8: noqa
def test_get_directories_for_bucket_with_prefix_recursively_works_on_bucket_nested_directory_with_three_directories_in_root():
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()

    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-1/a/b/c/test_fileobj.txt",
    )
    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-2/a/b/c/test_fileobj.txt",
    )
    s3_client.upload_fileobj(
        io.BytesIO(b"my data stored as file object in RAM"),
        bucket="test-bucket",
        key="test-directory-3/a/b/c/test_fileobj.txt",
    )

    directories = s3_client.get_directories_for_bucket_with_prefix_recursively(
        bucket="test-bucket", prefix="test-directory-3/"
    )

    assert sorted(directories) == [
        "test-directory-3/",
        "test-directory-3/a/",
        "test-directory-3/a/b/",
        "test-directory-3/a/b/c/",
    ]



from pathlib import Path
from uuid import uuid4

import boto3
from moto import mock_s3

from pys3thon.client import S3Client


@mock_s3
def test_persist_file(tmpdir):
    tmpdir = Path(tmpdir)
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    source_bucket = "test-bucket"
    source_key = f"{str(uuid4())}/Nepean.pdf"
    destination_bucket = "test-bucket"
    destination_key = f"{str(uuid4())}/Nepean.pdf"

    tmp_file_path = str(tmpdir / "Nepean.pdf")
    with open(tmp_file_path, "w") as file1:
        # Write some content to the file
        file1.write("Hello, world!")

    s3_client.upload_file(tmp_file_path, source_bucket, source_key)
    assert s3_client.check_if_exists_in_s3(source_bucket, source_key)

    s3_client.copy(
        source_bucket, source_key, destination_bucket, destination_key
    )
    assert s3_client.check_if_exists_in_s3(destination_bucket, destination_key)


@mock_s3
def test_persist_file_for_uri_encoded_file(tmpdir):
    tmpdir = Path(tmpdir)
    conn = boto3.resource("s3", region_name="ap-southeast-2")
    conn.create_bucket(
        Bucket="test-bucket",
        CreateBucketConfiguration={"LocationConstraint": "ap-southeast-2"},
    )
    s3_client = S3Client()
    source_bucket = "test-bucket"
    source_key = f"{str(uuid4())}/Nepean%20Legend%20(2).pdf"
    destination_bucket = "test-bucket"
    destination_key = f"{str(uuid4())}/Nepean%20Legend%20(2).pdf"

    tmp_file_path = str(tmpdir / "Nepean%20Legend%20(2).pdf")
    with open(tmp_file_path, "w") as file1:
        # Write some content to the file
        file1.write("Hello, world!")

    s3_client.upload_file(tmp_file_path, source_bucket, source_key)
    assert s3_client.check_if_exists_in_s3(source_bucket, source_key)

    s3_client.copy(
        source_bucket, source_key, destination_bucket, destination_key
    )
    assert s3_client.check_if_exists_in_s3(destination_bucket, destination_key)
